#include <stdio.h>
#include <time.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <getopt.h>

#include "vme_io.h"
void
usage_exit(const char *prgname)
{
  fprintf(stderr,"$Id: read_vme.c,v 1.12 1997/06/06 22:09:54 hannappe Exp $\nUsage: %s <options> <VME-Address> <bytes to read | values to write...> \n"
	  "\t-d <file>\tdevice file name (/dev/vme_a32_x)\n"
	  "\t-v\t\tbe more verbose\n"
	  "\t-h\t\tgive this (short) help\n"
	  "\t-b\t\tuse byte access\n"
	  "\t-w\t\tuse word access(default)\n"
	  "\t-l\t\tuse longword access\n"
	  "\t-L\t\tuse long long (64Bit) access\n"
	  "\t-s\t\tuse supervisory access\n"
	  "\t-u\t\tuse user access (default)\n"
	  "\t-1\t\tuse A16 (short IO) address space\n"
	  "\t-2\t\tuse A24 address space\n"
	  "\t-3\t\tuse A32 address space (default)\n"
	  "\t-i\t\tuse read()/write() io acesses\n"
	  "\t-m\t\tuse memory mapped access (default)\n"
	  "\t-q\t\tquiet mode, i.e. no output of values read\n"
	  "\t-B\t\tblock mode, i.e. read everything into a buffer\n"
	  "\t-W\t\twrite values to VMEBus\n"
	  "\t-R\t\tread values from VMEBus\n",
	  prgname);
  exit(1);
}

int main(int argc, char **argv)
{
  char *Device = NULL;
  int Verbose = 0;
  int Quiet = 0;
  int Blockmode  = 0;
  int Write = 0;
  int IO_access = 0;
  struct vme_mapping_ctrl Mapping = {
    VMEMAP_DWIDTH_16, VMEMAP_ASPACE_A32,
    VMEMAP_SUPUSERAM_USER, VMEMAP_PRGDATAAM_DATA};

  if (strstr(argv[0],"write_vme") != NULL)
    Write = 1;

  for (;;)
    {
      int c;
      c = getopt(argc, argv, "d:bwlLsu123WRimqBvh");
      switch(c)
	{
	case -1: break;
	case 'd': Device = optarg; break;
	case 'b': Mapping.max_datawidth = VMEMAP_DWIDTH_8; break;
	case 'w': Mapping.max_datawidth = VMEMAP_DWIDTH_16; break;
	case 'l': Mapping.max_datawidth = VMEMAP_DWIDTH_32; break;
	case 'L': Mapping.max_datawidth = VMEMAP_DWIDTH_64; break;
	case 's': Mapping.supuseram = VMEMAP_SUPUSERAM_SUPER; break;
	case 'u': Mapping.supuseram = VMEMAP_SUPUSERAM_USER; break;
	case '1': Mapping.address_space = VMEMAP_ASPACE_A16; break;
	case '2': Mapping.address_space = VMEMAP_ASPACE_A24; break;
	case '3': Mapping.address_space = VMEMAP_ASPACE_A32; break;
	case 'W': Write = 1; break;
	case 'R': Write = 0; break;
	case 'i': IO_access = 1; break;
	case 'm': IO_access = 0; break;
	case 'q': Quiet = 1; break;
	case 'B': Blockmode = 1; break;
	case 'v': Verbose = 1; break;
	case 'h':
	default:
	  usage_exit(argv[0]);
	}
      if (c== -1) break;
    }
  {
    unsigned int vme_addr,nof_bytes,granularity,gran_loss;
    int fd;
    
    volatile unsigned char *mapped_data;
    
    if (Write)
      {
	size_t datasize;
	switch (Mapping.max_datawidth) {
	case VMEMAP_DWIDTH_8: datasize = sizeof(char);
	case VMEMAP_DWIDTH_16: datasize = sizeof(short);
	case VMEMAP_DWIDTH_32: datasize = sizeof(int);
	case VMEMAP_DWIDTH_64: datasize = sizeof(int);
	}
	vme_addr = strtoul(argv[optind],NULL,0);
	nof_bytes = datasize * (argc - optind - 1);
      }
    else
      {
	if (argc != optind+2)
	  usage_exit(argv[0]);
	vme_addr = strtoul(argv[optind],NULL,0);
	nof_bytes = strtoul(argv[optind+1],NULL,0);
      }
    if (Device)
      {
	fd = open(Device,O_RDWR);
	if (fd == -1)
	  {
	    fprintf(stderr,"%s: Can't open device %s",
		    argv[0], Device);
	    perror("");
	    exit(1);
	  }
      }
    else
      {
	int i;
	for (i=0; i<4; i++)
	  {
	    char device[FILENAME_MAX];
	    sprintf(device,"/dev/vme_a32_%d",i);
	    fd = open(device, O_RDWR);
	    if (fd == -1)
	      {
		if (errno == EBUSY) continue;
		fprintf(stderr,"%s: can't open %s", argv[0],device);
		perror("");
	      }
	    else
	      break;
	  }
	if (fd == -1)
	  {
	    fprintf(stderr,"%s: No vme device available\n",argv[0]);
	    exit(1);
	  }
      }

    if (ioctl(fd, VMEIMG_SETMAPPING, &Mapping))
      {
	perror("ioctl VMEIMG_SETMAPPING");
	exit(1);
      }
    if (ioctl(fd, VMEIMG_SETVMEADDR, vme_addr))
      {
	perror("ioctl VMEIMG_SETVMEADDR");
	exit(1);
      }
    {
      unsigned long real_vme_addr;
      if (ioctl(fd, VMEIMG_GETVMEADDR, &real_vme_addr))
	{
	  perror("ioctl VMEIMG_GETVMEADDR");
	  exit(1);
	}
      if (ioctl(fd, VMEIMG_GETGRANULARITY, &granularity))
	{
	  perror("ioctl VMEIMG_GETVMEADDR");
	  exit(1);
	}
      gran_loss = vme_addr - real_vme_addr;
      if (Verbose)
	fprintf(stderr,
		"Granularity is 0x%x, giving an aligned vme addr of 0x%08lX\n",
		granularity, real_vme_addr);
    }
    if (Verbose)
      {
	char *amode, *dwidth, *sustat, *prgdat, *iomode;
	switch (Mapping.address_space) {
	case VMEMAP_ASPACE_A32: amode = "A32"; break;
	case VMEMAP_ASPACE_A24: amode = "A24"; break;
	case VMEMAP_ASPACE_A16: amode = "A16"; break;
	};
	switch (Mapping.max_datawidth) {
	case VMEMAP_DWIDTH_8: dwidth = "D08"; break;
	case VMEMAP_DWIDTH_16: dwidth = "D16"; break;
	case VMEMAP_DWIDTH_32: dwidth = "D32"; break;
	case VMEMAP_DWIDTH_64: dwidth = "D64"; break;
	};
	switch (Mapping.supuseram) {
	case VMEMAP_SUPUSERAM_USER: sustat = "user"; break;
	case VMEMAP_SUPUSERAM_SUPER: sustat = "supervisory"; break;
	};
	switch (Mapping.prgdataam) {
	case VMEMAP_PRGDATAAM_DATA: prgdat = "data"; break;
	case VMEMAP_PRGDATAAM_PRG: prgdat = "program"; break;
	};
	if (IO_access) 
	  iomode = "read()/write()";
	else
	  iomode = "memory mapped";
	fprintf(stderr,"Using %s/%s %s %s %s acesses\n",
		amode, dwidth, sustat, prgdat,iomode); 
      }
    
    if (!IO_access)
      {
	mapped_data = mmap(NULL,nof_bytes+gran_loss,
			   PROT_WRITE | PROT_READ, MAP_FILE|MAP_SHARED, fd,0);
	if (mapped_data == (volatile unsigned char*) -1)
	  {
	    perror("mmap()");
	    exit(1);
	  }
	fprintf(stderr,"Mapped adress is %p\n", mapped_data);
      }
    if (Blockmode)
      {
	if (Write)
	  {
	    struct timeval start,stop;
	    double t1,t2;
	    size_t bytes_written;
	    char *buffer;
	    if (Verbose)
	      fprintf(stderr,
		      "will write %u bytes to %p\n",
		      nof_bytes,(char *)vme_addr);
	    buffer = malloc(nof_bytes);
	    if (!buffer)
	      {
		perror("no buffer memory");
		exit(1);
	      }
	    {
	      unsigned char *ptr = buffer;
	      int i;
	      for (i=optind+1; i<argc; i++)
		switch (Mapping.max_datawidth) {
		case VMEMAP_DWIDTH_8:
		  {
		    unsigned char value, *hptr = ptr;
		    value = strtol(argv[i],NULL,0);
		    *hptr = value;
		    ptr+= sizeof(value);;
		    break;
		  }
		case VMEMAP_DWIDTH_16:
		  {
		    unsigned short value, *hptr = (unsigned short *)ptr;
		    value = strtol(argv[i],NULL,0);
		    value = htons(value);
		    *hptr = value;
		    ptr+= sizeof(value);;
		    break;
		  }
		case VMEMAP_DWIDTH_32:
		case VMEMAP_DWIDTH_64:
		  {
		    unsigned int value, *hptr = (unsigned int *)ptr;
		    value = strtol(argv[i],NULL,0);
		    value = htonl(value);
		    *hptr = value;
		    ptr+= sizeof(value);;
		    break;
		  }
		}
	    }
	    if (IO_access)
	      {
	 	gettimeofday(&start,NULL);
		lseek(fd, gran_loss, SEEK_SET);
		bytes_written = write (fd, buffer, nof_bytes);
		gettimeofday(&stop ,NULL);
		if (bytes_written != nof_bytes)
		  {
		    
		    fprintf(stderr,"write failed, only %d of %d bytes read",
			    bytes_written, nof_bytes);
		    perror("");
		    exit;
		  }
	      }
	    else
	      {
		gettimeofday(&start,NULL);
		memcpy(mapped_data + gran_loss,buffer,nof_bytes);
		gettimeofday(&stop ,NULL);
		bytes_written = nof_bytes;
	      }
	    t1 = start.tv_sec + 1e-6*start.tv_usec;
	    t2 = stop.tv_sec + 1e-6*stop.tv_usec;
	    fprintf(stderr,"%d bytes in %g usec, %gMB/s\n",
		    bytes_written, (t2-t1)*1e6,
		    bytes_written / ((t2-t1) *1024*1024));
	  }
	else
	  {
	    struct timeval start,stop;
	    double t1,t2;
	    size_t bytes_read;
	    char *buffer;
	    if (Verbose)
	      fprintf(stderr,
		      "will read %u bytes from %p on\n",
		      nof_bytes,(char *)vme_addr);

	    buffer = malloc(nof_bytes);
	    if (!buffer)
	      {
		perror("no buffer memory");
		exit(1);
	      }
	    if (IO_access)
	      {
		gettimeofday(&start,NULL);
		lseek(fd, gran_loss, SEEK_SET);
		bytes_read = read (fd, buffer, nof_bytes);
		gettimeofday(&stop ,NULL);
		if (bytes_read != nof_bytes)
		  {
		    
		    fprintf(stderr,"read failed, only %d of %d bytes read",
			    bytes_read, nof_bytes);
		    perror("");
		    exit;
		  }
	      }
	    else
	      {
		gettimeofday(&start,NULL);
		memcpy(buffer,mapped_data + gran_loss,nof_bytes);
		gettimeofday(&stop ,NULL);
		bytes_read = nof_bytes;
	      }
	    t1 = start.tv_sec + 1e-6*start.tv_usec;
	    t2 = stop.tv_sec + 1e-6*stop.tv_usec;
	    fprintf(stderr,"%d bytes in %g usec, %gMB/s\n",
		    bytes_read, (t2-t1)*1e6,
		    bytes_read / ((t2-t1) *1024*1024));
	  }
      }
    else
      if (Write)
	{
	  int i;
	  volatile unsigned char *ptr;
	  if (Verbose)
	    fprintf(stderr,
		    "will write %u bytes to %px\n",
		    nof_bytes,(char *)vme_addr);
	  
	  
	  ptr = mapped_data+gran_loss;
 	  for (i=optind+1; i<argc; i++)
	    {
	      switch (Mapping.max_datawidth) {
	      case VMEMAP_DWIDTH_8:
		{
		  unsigned char value;
		  volatile unsigned char *hptr;
		  value = strtol(argv[i],NULL,0);
		  if (Verbose)
		    fprintf(stderr,"Will write %02x to %08X",
			    value,(ptr - mapped_data) + vme_addr -gran_loss);
		  if (IO_access)
		    {
		      if (lseek(fd, ptr - mapped_data, SEEK_SET) == -1)
			{
			  perror("lseek failed");
			  exit(1);
			}
		      if (write(fd,&value,sizeof(value)) != sizeof(value))
			{
			  perror("write failed");
			  exit(1);
			}
		    }
		  else
		    {
		      hptr = (volatile unsigned char *)ptr;
		      *hptr = value;
		    }
		  if (Verbose) fprintf(stderr,".\n");
		  ptr++;
		  break;
		}
	      case VMEMAP_DWIDTH_16:
		{
		  unsigned short value;
		  volatile unsigned short *hptr;
		  value = strtol(argv[i],NULL,0);
		  if (Verbose)
		    fprintf(stderr,"Will write %04x to %08X",
			    value,(ptr - mapped_data) + vme_addr -gran_loss);
		  if (IO_access)
		    {
		      if (lseek(fd, ptr - mapped_data, SEEK_SET) == -1)
			{
			  perror("lseek failed");
			  exit(1);
			}
		      value = htons(value);
		      if (write(fd,&value,sizeof(value)) != sizeof(value))
			{
			  perror("write failed");
			  exit(1);
			}
		    }
		  else
		    {
		      hptr = (volatile unsigned short *)ptr;
		      *hptr = htons(value);
		    }
		  if (Verbose) fprintf(stderr,".\n");
		  ptr+=sizeof(value);
		  break;
		}
	      case VMEMAP_DWIDTH_32:
	      case VMEMAP_DWIDTH_64:
		{
		  unsigned int value;
		  volatile unsigned int *hptr;
		  value = strtol(argv[i],NULL,0);
		  if (Verbose)
		    fprintf(stderr,"Will write %08x to %08X",
			    value,(ptr - mapped_data) + vme_addr -gran_loss);
		  if (IO_access)
		    {
		      if (lseek(fd, ptr - mapped_data, SEEK_SET) == -1)
			{
			  perror("lseek failed");
			  exit(1);
			}
		      value = htonl(value);
		      if (write(fd,&value,sizeof(value)) != sizeof(value))
			{
			  perror("write failed");
			  exit(1);
			}
		    }
		  else
		    {
		      hptr = (volatile unsigned int *)ptr;
		      *hptr = htonl(value);
		    }
		  if (Verbose) fprintf(stderr,".\n");
		  ptr+=sizeof(value);
		  break;
		}
	      }
	    }
	}
      else
	{
	  volatile unsigned char *ptr;
	  struct timeval start,stop;
	  double t1,t2;
	  gettimeofday(&start,NULL);
	  if (Verbose)
	    fprintf(stderr,
		    "will read %u bytes from %p on\n",
		    nof_bytes,(char *)vme_addr);

	  for (ptr = mapped_data + gran_loss; 
	       ptr < mapped_data + gran_loss + nof_bytes;)
	    {
	      switch (Mapping.max_datawidth) {
	      case VMEMAP_DWIDTH_8:
		{
		  unsigned char value;
		  if (IO_access)
		    {
		      if (lseek(fd, ptr - mapped_data, SEEK_SET) == -1)
			{
			  perror("lseek failed");
			  exit(1);
			}
		      if (read(fd,&value,sizeof(value)) != sizeof(value))
			{
			  perror("read failed");
			  exit(1);
			}
		    }
		  else
		    value = *ptr;
		  printf("%08X %02x\n",ptr-mapped_data + vme_addr -gran_loss,
			 value);
		  ptr++;
		  break;
		}
	      case VMEMAP_DWIDTH_16:
		{
		  unsigned short value;
		  if (IO_access)
		    {
		      if (lseek(fd, ptr - mapped_data, SEEK_SET) == -1)
			{
			  perror("lseek failed");
			  exit(1);
			}
		      if (read(fd,&value,sizeof(value)) != sizeof(value))
			{
			  perror("read failed");
			  exit(1);
			}
		    }
		  else
		    value = *((volatile unsigned short *)ptr);
		  value = ntohs(value);
		  printf("%08X %04x\n",ptr-mapped_data + vme_addr -gran_loss,
			 value);
		  ((volatile unsigned short *)ptr)++;
		  break;
		}
	      case VMEMAP_DWIDTH_32:
	      case VMEMAP_DWIDTH_64:
		{
		  unsigned int value;
		  if (IO_access)
		    {
		      if (lseek(fd, ptr - mapped_data, SEEK_SET) == -1)
			{
			  perror("lseek failed");
			  exit(1);
			}
		      if (read(fd,&value,sizeof(value)) != sizeof(value))
			{
			  perror("read failed");
			  exit(1);
			}
		    }
		  else
		    value = *((volatile unsigned int *)ptr);
		  value = ntohl(value);
		  printf("%08X %08x\n",ptr-mapped_data + vme_addr -gran_loss,
			 value);
		  ((volatile unsigned int *)ptr)++;
		  break;
		}
	      }
	    }
	  gettimeofday(&stop,NULL);
	  t1 = start.tv_sec + 1e-6*start.tv_usec;
	  t2 = stop.tv_sec + 1e-6*stop.tv_usec;
	  fprintf(stderr,"%d accesses lasted %g usec\n",nof_bytes, (t2-t1)*1e6);
	}
    munmap(mapped_data,nof_bytes);
    close(fd);
  }
  exit(0);
}


/*
 * Local variables:
 * compile-command: "gcc -o read_vme -Wall read_vme.c"
 * End:
 */
